# -*- coding: utf-8 -*-

"""
    Copyright 2010 cloudControl UG (haftungsbeschraenkt)

    Licensed under the Apache License, Version 2.0 (the "License");
    you may not use this file except in compliance with the License.
    You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

    Unless required by applicable law or agreed to in writing, software
    distributed under the License is distributed on an "AS IS" BASIS,
    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
    See the License for the specific language governing permissions and
    limitations under the License.
"""
import os
import sys
import commands

from subprocess import call
from error import InputErrorException
from oshelpers import readContentOf, isValidFile


def is_key_valid(key):
    """
        Is the given key a valid SSH-key with RSA encryption?
    """
    if not isValidFile(key):
        return False

    file_content = readContentOf(key)
    if file_content == None:
        return False

    # File read! Now check if it's RSA encrypted ...
    if file_content[:8] != 'ssh-rsa ':
        return False

    return True


def generate_rsa_keys():
    """
        Create new set of SSH keys via shell with 'ssh-keygen'
        or, for win32, via paramiko
    """
    if sys.platform == 'win32':
        ssh_path = os.path.expanduser('~') + "/.ssh"
    else:
        ssh_path = os.getenv("HOME") + "/.ssh"

    # If we're on Windows, we need to take a different approach
    if sys.platform == 'win32':
        return generate_rsa_key_manually(ssh_path)

    # Check if default keys already exist. If yes, bail out!
    if os.path.exists(ssh_path + "/id_rsa.pub"):
        return False

    # Check if "ssh-keygen" is installed. If not, stop right here!
    error_code = commands.getstatusoutput("which ssh-keygen")[0]
    if error_code != 0:
        return False

    # Call "ssh-keygen" to let the users create his/her keys ...
    call(["ssh-keygen", "-t", "rsa", "-b", "2048", "-f", ssh_path + "/id_rsa"])

    return True


def generate_rsa_key_manually(user_ssh_path, key_file_name="id_rsa.pub"):
    """
        Generate an RSA-encrypted key for the user
        Will return public key string if all went well, otherwise "None"
    """
    if key_file_name == "id_rsa.pub":
        if not os.path.exists(user_ssh_path):
            os.mkdir(user_ssh_path, 0700)

    # Create the private key, first.
    key_base_name = key_file_name.rstrip(".pub")
    private_key = generate_private_rsa_key_file(user_ssh_path, key_base_name)

    # Now, create the public key (using the private key) ...
    generate_public_rsa_key_file(private_key, user_ssh_path, key_base_name)

    return True


def generate_private_rsa_key_file(ssh_path, key_base_name):
    """
        Generate a default PRIVATE SSH key file using RSA
    """
    # paramiko uses pycrypto's RandomPool which throws a deprecation warning
    import warnings
    with warnings.catch_warnings():
        warnings.simplefilter("ignore")
        from paramiko import RSAKey

    key = RSAKey.generate(2048)
    private_key_file_name = ssh_path + "/" + key_base_name
    key.write_private_key_file(private_key_file_name)

    # Ok, pass back private key object
    return key


def generate_public_rsa_key_file(private_key, ssh_path, key_base_name):
    """
        Generate a default PUBLIC SSH key file using RSA
    """
    try:
        username = os.getlogin()
    except AttributeError:
        username = os.getenv('USERNAME', 'cloudControl')

    try:
        hostname = os.uname()[1]
    except AttributeError:
        hostname = os.getenv('COMPUTERNAME', 'localhost')

    # Create the content of the public key as string
    pubkey_string = 'ssh-rsa {0} {1}@{2} \n'.format(
        private_key.get_base64(),
        username,
        hostname)

    # Write the public key to file system
    public_key_filename = ssh_path + "/" + key_base_name + ".pub"
    pubkey = open(public_key_filename, 'w')
    pubkey.write(pubkey_string)
    pubkey.close()

    return True


def create_new_default_ssh_keys():
    """
        Generate a set of private and public keys for user
    """
    question = raw_input('Type "Yes" to generate a new default SSH-key pair: ')
    if question.lower() != 'yes':
        raise InputErrorException('SecurityQuestionDenied')

    # Let user create key, then check if everything went fine!
    if not generate_rsa_keys():
        raise InputErrorException('UserShouldCreateKey')

    # Ok, new SSH default keys seemed to be created.
    return True


def ask_user_to_use_default_ssh_public_key():
    """
        Ask the user if the default public SSH-key (RSA)
        shall be used.
    """
    if sys.platform == 'win32':
        default_rsa_public_key = os.path.expanduser('~')
    else:
        default_rsa_public_key = os.getenv("HOME") + "/.ssh/id_rsa.pub"

    # Check first if we actually have a default SSH public key.
    # If we don't then simply return nothing ("")
    if not isValidFile(default_rsa_public_key):
        return ""

    # Ok, found! Ask user if we should use this one ...
    question = raw_input("Found default key '{0}' . ".format(default_rsa_public_key) +
                         'Type "Yes" to use the default key: ')
    if question.lower() != 'yes':
        raise InputErrorException('SecurityQuestionDenied')

    return 0



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
    Copyright 2014 cloudControl GmbH

    Licensed under the Apache License, Version 2.0 (the "License");
    you may not use this file except in compliance with the License.
    You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

    Unless required by applicable law or agreed to in writing, software
    distributed under the License is distributed on an "AS IS" BASIS,
    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
    See the License for the specific language governing permissions and
    limitations under the License.
"""

import argparse

from cctrl.settings import VERSION
from pycclib.cclib import *
from pycclib.version import __version__ as cclibversion
import cctrl.common as common
from cctrl.app import AppController, AppsController
from cctrl.output import get_version


class list_action(argparse.Action):
    """
        This action is needed by argparse because we use --list to get a list
        of apps that a user has access too. This is not directly supported by
        one of the other argpase actions. Therefore we needed to write our own.

        #TODO: Right now we do the error handling here and in common.run.
        It would be better to use common.run here too.
    """

    def __init__(self, api, **kwargs):
        super(list_action, self).__init__(**kwargs)
        self.api = api

    def __call__(self, parser, namespace, value, option_string=None):
        try:
            common.check_for_updates(self.api.check_versions()['cctrl'])
        except KeyError:
            pass
        except ConnectionException:
            pass

        apps = AppsController(self.api)
        common.execute_with_authenticated_user(self.api, lambda: apps.list())
        parser.exit()


def parse_cmdline(app):
    """
        We parse the commandline using argparse from
        http://code.google.com/p/argparse/.

        argparse works with parsers and subparsers.

        Our main parser for cctrlapp always requires a name argument. We then
        add subparses for each action available for the app identified by name.

        Each subparser then may have it's own arguments needed for that
        particular subparsers action to work.
    """
    version = get_version(VERSION, cclibversion)
    description = "%(prog)s controls your cloudControl apps"
    epilog = "And now you're in control"

    parser = argparse.ArgumentParser(description=description, epilog=epilog)

    list_apps = parser.add_argument_group(
        'list apps',
        'get a list of all your apps')
    list_apps.add_argument(
        '-l',
        '--list',
        action=list_action,
        nargs=0,
        api=app.api,
        help='list apps')

    control_apps = parser.add_argument_group(
        'control apps',
        'control app or deployment')

    control_apps.add_argument(
        'name',
        help='app_name for apps or app_name/deployment_name for deployments',
        metavar='app_name[/deployment_name]')

    subparsers = parser.add_subparsers(
        title='commands',
        description='available commands',
        metavar='')

    run_cmd_subparser = subparsers.add_parser(
        'run',
        help="run command")
    run_cmd_subparser.add_argument(
        'command',
        help='command to execute')
    run_cmd_subparser.set_defaults(func=app.run_cmd)

    rollback_cmd_subparser = subparsers.add_parser(
        'rollback',
        help="rollback to previous deployment")
    rollback_cmd_subparser.set_defaults(func=app.rollback_cmd)

    create_subparser = subparsers.add_parser(
        'create',
        help="create new application")
    create_subparser.add_argument(
        'type',
        help='app-type (language)',
        metavar='type')
    create_subparser.add_argument(
        '--buildpack',
        help='custom buildpack URL (for "custom" app type)',
        default=None,
        metavar='url')
    create_subparser.add_argument(
        '--repo',
        choices=['bzr', 'git'],
        help='bzr or git',
        default=None,
        metavar='repo')
    create_subparser.set_defaults(func=app.create)

    details_subparser = subparsers.add_parser(
        'details',
        help="app or deployment details")
    details_subparser.set_defaults(func=app.details)

    delete_subparser = subparsers.add_parser(
        'delete',
        help="delete application")
    delete_subparser.add_argument(
        '-f',
        '--force',
        action="store_true",
        dest="force_delete",
        help="don't ask for confirmation")
    delete_subparser.set_defaults(func=app.delete)

    push_subparser = subparsers.add_parser('push', help="push local changes")
    push_subparser.add_argument(
        'source',
        nargs='?',
        help='path to local branch')
    push_subparser.add_argument(
        '--ship',
        action='store_true',
        dest='ship',
        help='deploy, show logs and open last version in browser after pushing')
    push_subparser.add_argument(
        '--clear-buildpack-cache',
        action='store_true',
        dest='clear_cache',
        help="clear the deployment buildpack cache so all dependencies are downloaded again")
    push_subparser.add_argument(
        '--deploy',
        action='store_true',
        dest='deploy',
        help='deploy after pushing')
    push_subparser.set_defaults(func=app.push)

    deploy_subparser = subparsers.add_parser('deploy', help="deploy version")
    deploy_subparser.add_argument(
        '--containers', '--min',
        type=int,
        metavar='NUM',
        help="number of containers")
    deploy_subparser.add_argument(
        '--size', '--max',
        type=int,
        help="container memory size in units of 128MB")
    deploy_subparser.add_argument(
        '--memory',
        help="container size in MB, --memory=X*128 maps to --size=X")
    deploy_subparser.add_argument(
        '--stack',
        action="store",
        nargs='?',
        default=None,
        dest="stack",
        help="desired stack name")
    deploy_subparser.add_argument(
        'version',
        nargs='?',
        default=-1,
        help='optionally accepts a version')
    deploy_subparser.add_argument(
        '--restart-workers',
        action='store_true',
        dest='restart_workers',
        help='restart the worker after deploying')
    push_subparser.set_defaults(func=app.push)
    deploy_subparser.set_defaults(func=app.deploy)

    undeploy_subparser = subparsers.add_parser(
        'undeploy',
        help="delete deployment")
    undeploy_subparser.add_argument(
        '-f',
        '--force',
        action="store_true",
        dest="force_delete",
        help="don't ask for confirmation")
    undeploy_subparser.set_defaults(func=app.undeploy)

    showUser_subparser = subparsers.add_parser('user', help="list app users")
    showUser_subparser.set_defaults(func=app.showUser)

    addUser_subparser = subparsers.add_parser(
        'user.add',
        help="add user by email")
    addUser_subparser.add_argument('email')
    addUser_subparser.add_argument(
        '--role',
        action="store",
        nargs='?',
        default=None,
        dest="role",
        help="role")
    addUser_subparser.set_defaults(func=app.addUser)

    removeUser_subparser = subparsers.add_parser(
        'user.remove',
        help="remove user by username or email")
    removeUser_subparser.add_argument('username', help='username or email')
    removeUser_subparser.set_defaults(func=app.removeUser)

    showConfig_subparser = subparsers.add_parser('config', help="list config vars")
    showConfig_subparser.add_argument(
        'key',
        nargs='?',
        help='name of the key e.g. foo')
    showConfig_subparser.set_defaults(func=app.showConfig)

    addConfig_subparser = subparsers.add_parser(
        'config.add',
        help="add config vars")
    addConfig_subparser.add_argument(
        '-f',
        '--force',
        action="store_true",
        dest="force_add",
        help="don't ask for confirmation")
    addConfig_subparser.add_argument(
        'variables',
        nargs=argparse.REMAINDER,
        help='variables to add passed through to the config add-on e.g foo=bar baz=faa')
    addConfig_subparser.set_defaults(func=app.addConfig)

    removeConfig_subparser = subparsers.add_parser(
        'config.remove',
        help="remove config variables")
    removeConfig_subparser.add_argument(
        'variables',
        nargs=argparse.REMAINDER,
        help='variables to remove on the config add-on e.g. foo baz')
    removeConfig_subparser.set_defaults(func=app.removeConfig)

    showAddon_subparser = subparsers.add_parser('addon', help="show addon")
    showAddon_subparser.add_argument(
        'addon',
        nargs='?',
        help='name of the addon e.g. mysql.free')
    showAddon_subparser.set_defaults(func=app.showAddon)

    showAddon_subparser = subparsers.add_parser(
        'addon.list',
        help="list all available addons")
    showAddon_subparser.set_defaults(func=app.listAddons)

    showAddon_subparser = subparsers.add_parser(
        'addon.creds',
        help="print creds.json format to stdout")
    showAddon_subparser.add_argument(
        'addon',
        nargs='?',
        help='name of the addon e.g. mysql.free')
    showAddon_subparser.set_defaults(func=app.showAddonCreds)

    addAddon_subparser = subparsers.add_parser(
        'addon.add',
        help="add addon")
    addAddon_subparser.add_argument(
        'addon',
        help='name of the addon e.g. mysql.free')
    addAddon_subparser.add_argument(
        'options',
        nargs=argparse.REMAINDER,
        help='options passed through to the add-on - if path to file the content is sent')
    addAddon_subparser.set_defaults(func=app.addAddon)

    addAddon_subparser = subparsers.add_parser(
        'addon.upgrade',
        help="upgrade addon")
    addAddon_subparser.add_argument(
        'addon_old',
        help='current addon option')
    addAddon_subparser.add_argument(
        'addon_new',
        help='addon option to upgrade to')
    addAddon_subparser.set_defaults(func=app.updateAddon)

    addAddon_subparser = subparsers.add_parser(
        'addon.downgrade',
        help="downgrade addon")
    addAddon_subparser.add_argument(
        'addon_old',
        help='current addon option')
    addAddon_subparser.add_argument(
        'addon_new',
        help='addon option to downgrade to')
    addAddon_subparser.set_defaults(func=app.updateAddon)

    removeAddon_subparser = subparsers.add_parser(
        'addon.remove',
        help="remove addon")
    removeAddon_subparser.add_argument(
        'addon',
        help='name of the addon e.g. mysql.free')
    removeAddon_subparser.set_defaults(func=app.removeAddon)

    showAlias_subparser = subparsers.add_parser('alias', help="show alias")
    showAlias_subparser.add_argument(
        'alias',
        nargs='?',
        help='name of the alias e.g. www.example.com')
    showAlias_subparser.set_defaults(func=app.showAlias)

    addAlias_subparser = subparsers.add_parser('alias.add', help="add alias")
    addAlias_subparser.add_argument(
        'alias',
        help='name of the alias e.g. www.example.com')
    addAlias_subparser.set_defaults(func=app.addAlias)

    removeAlias_subparser = subparsers.add_parser(
        'alias.remove',
        help="remove alias")
    removeAlias_subparser.add_argument(
        'alias',
        help='name of the alias e.g. www.example.com')
    removeAlias_subparser.set_defaults(func=app.removeAlias)

    showWorker_subparser = subparsers.add_parser(
        'worker',
        help="show worker")
    showWorker_subparser.add_argument(
        'wrk_id',
        nargs='?',
        help='optional wrk_id for single worker details')
    showWorker_subparser.set_defaults(func=app.showWorker)

    addWorker_subparser = subparsers.add_parser(
        'worker.add',
        help="start worker")
    addWorker_subparser.add_argument(
        'command',
        help='the command to execute')
    addWorker_subparser.add_argument(
        'params',
        nargs='?',
        default=None,
        help='optional parameters to append to the command')
    addWorker_subparser.add_argument(
        '--size',
        type=int,
        help="container memory size in units of 128MB")
    addWorker_subparser.add_argument(
        '--memory',
        help="container size in MB, --memory=X*128 maps to --size=X")
    addWorker_subparser.set_defaults(func=app.addWorker)

    removeWorker_subparser = subparsers.add_parser(
        'worker.remove',
        help="stop worker")
    removeWorker_subparser.add_argument(
        'wrk_id',
        help='stop worker by wrk_id')
    removeWorker_subparser.set_defaults(func=app.removeWorker)

    restartWorker_subparser = subparsers.add_parser(
        'worker.restart',
        help="restart worker")
    restart_worker_group = restartWorker_subparser.add_mutually_exclusive_group(required=True)
    restart_worker_group.add_argument(
        'wrk_id',
        help='restart worker by wrk_id',
        nargs='?'
    )
    restart_worker_group.add_argument(
        '--all',
        action='store_true',
        dest='all',
        help='restart all currently running workers')
    restart_worker_group.set_defaults(func=app.restartWorker)

    showCron_subparser = subparsers.add_parser('cron', help="show cronjobs")
    showCron_subparser.add_argument(
        'job_id',
        nargs='?',
        help='optional job_id for single worker details')
    showCron_subparser.set_defaults(func=app.showCron)

    addCron_subparser = subparsers.add_parser('cron.add', help="add cronjob")
    addCron_subparser.add_argument('url', help='url to call')
    addCron_subparser.set_defaults(func=app.addCron)

    removeCron_subparser = subparsers.add_parser(
        'cron.remove',
        help="remove cronjob")
    removeCron_subparser.add_argument(
        'job_id',
        help='remove cronjob by job_id')
    removeCron_subparser.set_defaults(func=app.removeCron)

    log_subparser = subparsers.add_parser('log', help="show the log")
    log_subparser.add_argument(
        'type',
        choices=['access', 'error', 'worker', 'deploy'])
    log_subparser.add_argument(
        '-w',
        '--wrk_id',
        action="store",
        nargs='?',
        default=None,
        dest="wrk_id",
        help="a wrk_id to filter the worker log")
    log_subparser.add_argument(
        '-g',
        '--grep',
        action="store",
        nargs='?',
        default=None,
        dest="filter",
        help="pattern to search for within the log entry")
    log_subparser.set_defaults(func=app.log)

    parser.add_argument(
        '-v',
        '--version',
        action='version',
        version=version,
        help='show version string')

    open_subparser = subparsers.add_parser(
        'open',
        help="open deployment on default browser"
    )
    open_subparser.set_defaults(func=app.open)

    args = parser.parse_args()

    common.run(args, app.api)


def setup_cli(settings):
    api = common.init_api(settings)
    try:
        app = AppController(api, settings)
        parse_cmdline(app)
    except KeyboardInterrupt:
        pass
    finally:
        common.shutdown(api)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
    Copyright 2014 cloudControl GmbH

    Licensed under the Apache License, Version 2.0 (the "License");
    you may not use this file except in compliance with the License.
    You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

    Unless required by applicable law or agreed to in writing, software
    distributed under the License is distributed on an "AS IS" BASIS,
    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
    See the License for the specific language governing permissions and
    limitations under the License.
"""

import os

import argparse

from pycclib.version import __version__ as cclibversion

import cctrl.common as common
from cctrl.output import get_version
from cctrl.settings import VERSION
from cctrl.user import UserController


def parse_cmdline(user):
    """
      We parse the commandline using argparse from
      http://code.google.com/p/argparse/.
    """
    version = get_version(VERSION, cclibversion)
    usage = '%(prog)s [-h, --help] [command]'
    description = '%(prog)s controls your user account'
    epilog = "And now you're in control"

    parser = argparse.ArgumentParser(
        description=description,
        epilog=epilog,
        usage=usage)

    parser.add_argument('-v', '--version', action='version', version=version)

    subparsers = parser.add_subparsers(
        title='commands',
        description='available commands',
        metavar='')

    create_subparser = subparsers.add_parser('create', help="create user")
    create_subparser.add_argument(
        "--name",
        nargs=1,
        action="store",
        dest="name",
        help="the username")
    create_subparser.add_argument(
        "--email",
        nargs=1,
        action="store",
        dest="email",
        help="the email")
    create_subparser.add_argument(
        "--password",
        nargs=1,
        action="store",
        dest="password",
        help="the password")
    create_subparser.set_defaults(func=user.create)

    activate_subparser = subparsers.add_parser(
        'activate',
        help="activate user")
    activate_subparser.add_argument(
        'user_name',
        nargs=1,
        help='your user name')
    activate_subparser.add_argument(
        'activation_code',
        nargs=1,
        help='activation code from the email')
    activate_subparser.set_defaults(func=user.activate)

    delete_subparser = subparsers.add_parser('delete', help="delete user")
    delete_subparser.add_argument(
        '-f',
        '--force',
        action="store_true",
        dest="force_delete",
        help="don't ask for confirmation")
    delete_subparser.set_defaults(func=user.delete)

    listUsers_subparser = subparsers.add_parser(
        'key',
        help="list public keys and key ids")
    listUsers_subparser.add_argument(
        'id',
        nargs='?',
        help='print key by id')
    listUsers_subparser.set_defaults(func=user.listKeys)

    addKey_subparser = subparsers.add_parser('key.add', help="add public key")
    addKey_subparser.add_argument(
        'public_key',
        nargs='?',
        default=os.path.expanduser('~/.ssh/id_rsa.pub'),
        help='path to id_rsa.pub file')
    addKey_subparser.set_defaults(func=user.addKey)

    removeKey_subparser = subparsers.add_parser(
        'key.remove',
        help="remove public key")
    removeKey_subparser.add_argument(
        '-f',
        '--force',
        action="store_true",
        dest="force_delete",
        help="don't ask for confirmation")
    removeKey_subparser.add_argument(
        'id',
        nargs=1,
        help='the key_id of the key - see listKeys')
    removeKey_subparser.set_defaults(func=user.removeKey)

    logout_subparser = subparsers.add_parser(
        'logout',
        help="logout - this deletes the saved token")
    logout_subparser.set_defaults(func=user.logout)

    check_token_subparser = subparsers.add_parser('checktoken')
    check_token_subparser.set_defaults(func=user.checktoken)

    registerAddon_subparser = subparsers.add_parser('addon.register', help="registers an addon")
    registerAddon_subparser.add_argument(
        'manifest',
        help='path to the manifest file')
    registerAddon_subparser.set_defaults(func=user.registerAddon)

    args = parser.parse_args()

    common.run(args, user.api)


def setup_cli(settings):
    api = common.init_api(settings)
    try:
        user = UserController(api, settings)
        parse_cmdline(user)
    except KeyboardInterrupt:
        pass
    finally:
        common.shutdown(api)



# -*- coding: utf-8 -*-
"""
    Copyright 2010 cloudControl UG (haftungsbeschraenkt)

    Licensed under the Apache License, Version 2.0 (the "License");
    you may not use this file except in compliance with the License.
    You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

    Unless required by applicable law or agreed to in writing, software
    distributed under the License is distributed on an "AS IS" BASIS,
    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
    See the License for the specific language governing permissions and
    limitations under the License.
"""

import sys
import os

from cctrl.settings import VERSION

from pycclib import cclib
from cctrl.error import InputErrorException, messages
from cctrl.auth import get_credentials, update_tokenfile, delete_tokenfile, read_tokenfile
from cctrl.app import ParseAppDeploymentName


def check_for_updates(latest_version_str, our_version_str=VERSION):
    """
        check if the API reports a version that is greater then our currently
        installed one
    """
    our = dict()
    latest = dict()
    for version, suffix in ((our, our_version_str), (latest, latest_version_str)):
        for part in ['major', 'minor', 'patch']:
            version[part], _, suffix = suffix.partition('.')
            version[part] = int(version[part])
        version['suffix'] = suffix

    for part in ['major', 'minor', 'patch', 'suffix']:
        if latest[part] > our[part]:
            if part == 'major':
                sys.exit(messages['UpdateRequired'])
            else:
                print >> sys.stderr, messages['UpdateAvailable']
            return


def init_api(settings):
    """
        This methods initializes the API but first checks for a
        CCTRL_API_URL environment variable and uses it if found.
        For Windows we also need to load ca_certs differently,
        because the httplib2 provided ones are not included due to
        py2exe.
    """
    dirname = os.path.dirname(__file__)
    while len(dirname) > 1:
        p = os.path.join(dirname, 'cacerts.txt')

        if os.path.exists(p):
            cclib.CA_CERTS = p
            break

        dirname = os.path.dirname(dirname)
    return cclib.API(token=read_tokenfile(), url=settings.api_url, token_source_url=settings.token_source_url, register_addon_url=settings.register_addon_url, encode_email=settings.encode_email)


def get_email_and_password():
    # check ENV for credentials first
    try:
        email = os.environ.pop('CCTRL_EMAIL')
        password = os.environ.pop('CCTRL_PASSWORD')
    except KeyError:
        email, password = get_credentials()
    return email, password


def execute_with_authenticated_user(api, command):
    while True:
        try:
            try:
                command()
            except (cclib.TokenRequiredError, cclib.UnauthorizedError):
                email, password = get_email_and_password()
                try:
                    api.create_token(email, password)
                except cclib.UnauthorizedError:
                    sys.exit(messages['NotAuthorized'])
                else:
                    pass
            except ParseAppDeploymentName:
                sys.exit(messages['InvalidAppOrDeploymentName'])
            else:
                break
        except cclib.ForbiddenError, e:
            sys.exit(messages['NotAllowed'])
        except cclib.ConnectionException:
            sys.exit(messages['APIUnreachable'])
        except (cclib.BadRequestError, cclib.ConflictDuplicateError, cclib.GoneError,
                cclib.InternalServerError, cclib.NotImplementedError, cclib.ThrottledError,
                InputErrorException), e:
            sys.exit(e)


def run(args, api):
    """
        run takes care of calling the action with the needed arguments parsed
        using argparse.

        We first try to call the action. In case the called action requires a
        valid token and the api instance does not have one a TokenRequiredError
        gets raised. In this case we catch the error and ask the user for a
        email/password combination to create a new token. After that we call
        the action again.

        pycclib raises an exception any time the API does answer with a
        HTTP STATUS CODE other than 200, 201 or 204. We catch these exceptions
        here and stop cctrlapp using sys.exit and show the error message to the
        user.
    """

    execute_with_authenticated_user(api, (lambda: args.func(args)))


def shutdown(api):
    """
        shutdown handles updating or deleting the token file on disk each time
        cctrl finishes or gets interrupted.
    """
    if api.check_token():
        update_tokenfile(api)
    else:
        delete_tokenfile()



# -*- coding: utf-8 -*-
"""
    Copyright 2010 cloudControl UG (haftungsbeschraenkt)

    Licensed under the Apache License, Version 2.0 (the "License");
    you may not use this file except in compliance with the License.
    You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

    Unless required by applicable law or agreed to in writing, software
    distributed under the License is distributed on an "AS IS" BASIS,
    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
    See the License for the specific language governing permissions and
    limitations under the License.
"""

import os
import sys
import json

from cctrl.error import PasswordsDontMatchException, InputErrorException, messages
from cctrl.auth import get_credentials
from pycclib.cclib import GoneError, NotImplementedError, ForbiddenError
from cctrl.output import print_keys
from pycclib.cclib import ConflictDuplicateError
from output import print_key
from oshelpers import readContentOf
from keyhelpers import is_key_valid, ask_user_to_use_default_ssh_public_key, \
    create_new_default_ssh_keys
from pycclib import cclib
from cctrl.common import get_email_and_password


class UserController(object):
    """
        This controller handles all user related actions.
    """

    api = None

    def __init__(self, api, settings):
        self.api = api
        self.settings = settings

    def checktoken(self, args):
        try:
            self.api.read_users()
        except cclib.TokenRequiredError:
            sys.exit(1)
        sys.exit(0)

    def create(self, args):
        """
            Create a new user.
        """
        if not self.settings.user_registration_enabled:
            print messages['RegisterDisabled'].format(self.settings.user_registration_url)
            return

        self.api.set_token(None)
        if args.name and args.email and args.password:
            name = args.name[0]
            email = args.email[0]
            password = args.password[0]
        else:
            name = raw_input('Username: ')
            try:
                email, password = get_credentials(create=True)
            except PasswordsDontMatchException:
                return
        try:
            self.api.create_user(name, email, password)
        except NotImplementedError:
            raise InputErrorException('CommandNotImplemented')

        print messages['UserCreatedNowCheckEmail']

    def activate(self, args):
        """
            Activate a new user using the information from the
            activation email.
        """
        self.api.set_token(None)
        try:
            self.api.update_user(
                args.user_name[0],
                activation_code=args.activation_code[0])
        except GoneError:
            raise InputErrorException('WrongUsername')
        except NotImplementedError:
            raise InputErrorException('CommandNotImplemented')

    def delete(self, args):
        """
            Delete your user account.
        """

        users = self.api.read_users()
        if not args.force_delete:
            question = raw_input('Do you really want to delete your user? ' +
                                 'Type "Yes" without the quotes to delete: ')
        else:
            question = 'Yes'
        if question.lower() == 'yes':
            try:
                self.api.delete_user(users[0]['username'])
            except NotImplementedError:
                raise InputErrorException('CommandNotImplemented')
            except ForbiddenError:
                raise InputErrorException('DeleteAppsBeforeUser')

            # After we have deleted our user we should also delete
            # the token_file to avoid confusion
            self.api.set_token(None)
        else:
            raise InputErrorException('SecurityQuestionDenied')

    def addKey(self, args):
        """
            Add a given public key to cloudControl user account.
        """
        if sys.platform == 'win32':
            default_key_path = os.path.expanduser('~') + "/.ssh/id_rsa.pub"
        else:
            default_key_path = os.getenv("HOME") + "/.ssh/id_rsa.pub"

        # Possibility #1: User is providing a non-default SSH key
        key_to_read = args.public_key
        if not is_key_valid(key_to_read):

            # Possibility #2: Try the default RSA public key
            print "Key '{0}' seems to be invalid or not found!".format(key_to_read)
            ask_user_to_use_default_ssh_public_key()

            # Possibility #3: All failed! Let's just create new keys for user!
            if not is_key_valid(default_key_path):
                if key_to_read != default_key_path:
                    print "Default key '{0}' seems to be invalid or not found!".format(default_key_path)
                create_new_default_ssh_keys()

            # We've filtered all cases: the key must be the default one!
            key_to_read = default_key_path

        # Good, we have the key! Now, read the content of the key!
        public_rsa_key_content = readContentOf(key_to_read)

        # Add public RSA-key to cloudControl user account
        try:
            users = self.api.read_users()
            self.api.create_user_key(
                users[0]['username'],
                public_rsa_key_content)

        except ConflictDuplicateError:
            raise InputErrorException('KeyDuplicate')

    def listKeys(self, args):
        """
            List your public keys.
        """
        users = self.api.read_users()
        if args.id:
            key = self.api.read_user_key(users[0]['username'], args.id)
            print_key(key)
        else:
            keys = self.api.read_user_keys(users[0]['username'])
            print_keys(keys)

    def removeKey(self, args):
        """
            Remove one of your public keys specified by key_id.

            listKeys() shows the key_ids.
        """
        users = self.api.read_users()
        if not args.force_delete:
            question = raw_input('Do you really want to remove your key? ' +
                                 'Type "Yes" without the quotes to remove: ')
        else:
            question = 'Yes'
        if question.lower() == 'yes':
            self.api.delete_user_key(users[0]['username'], args.id[0])
        else:
            raise InputErrorException('SecurityQuestionDenied')

    def logout(self, args):
        """
            Logout a user by deleting the token.json file.
        """
        self.api.set_token(None)

    def registerAddon(self, args):
        file_content = readContentOf(args.manifest)
        email, password = get_email_and_password()
        self.api.register_addon(email, password, json.loads(file_content))



# -*- coding: utf-8 -*-

"""
    Copyright 2010 cloudControl UG (haftungsbeschraenkt)

    Licensed under the Apache License, Version 2.0 (the "License");
    you may not use this file except in compliance with the License.
    You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

    Unless required by applicable law or agreed to in writing, software
    distributed under the License is distributed on an "AS IS" BASIS,
    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
    See the License for the specific language governing permissions and
    limitations under the License.
"""
import os
import subprocess
import sys
import shlex

from cctrl.error import InputErrorException


def isValidFile(filename):
        """
            Is the given filename a valid file?
        """
        return os.path.isfile(os.path.abspath(filename))


def readContentOf(filename):
    """
        Read a given file's content into a string
        Returns contents of given file as string, otherwise "None"
    """
    file_content = ''

    if not os.path.isfile(os.path.abspath(filename)):
        raise InputErrorException('FileNotFound')

    try:
        open_file = open(os.path.abspath(filename), 'r')
        file_content = str(open_file.read())
    except IOError:
        raise InputErrorException('FileReadOrWriteFailed')

    return file_content.strip()


def which(programs):
    """
        from http://stackoverflow.com/questions/377017/ \
        test-if-executable-exists-in-python/377028#377028
    """
    def is_exe(file_path):
        return os.path.exists(file_path) and os.access(file_path, os.X_OK)

    for program in programs:
        file_path, file_name = os.path.split(program)
        if file_path:
            if is_exe(program):
                return program
        else:
            for path in os.environ["PATH"].split(os.pathsep):
                exe_file = os.path.join(path, program)
                if is_exe(exe_file):
                    return exe_file

    return None


def check_installed_rcs(name):
    """
        Check if either "bzr" or "git" is installed (and can be found
        via PATH variable)
    """
    rcs_executables = {
        'bzr': ['bzr.exe', 'bzr.bat', 'bzr'],
        'git': ['git', 'git.exe', 'git.cmd']}
    return which(rcs_executables[name])


def is_buildpack_url_valid(buildpack_url):
    """
        Is the given url a valid buildpack url?
    """
    try:
        branch = buildpack_url.split('#')[1]
    except IndexError:
        branch = 'master'
    sp = subprocess.Popen(
        ['git', 'ls-remote', buildpack_url.split('#')[0], branch],
        stdout=subprocess.PIPE,
        stderr=subprocess.PIPE,
        shell=False
    )
    stdout, _ = sp.communicate()
    valid = True if sp.returncode == 0 and 'refs/heads' in stdout else False
    return valid


def recode_input(input):
    return input.decode(sys.stdin.encoding or 'UTF-8').encode('UTF-8')


def ssh_cmd(host_name, *args):
    sshopts = shlex.split(os.environ.get('CCTRL_SSHOPTS', ''))
    return ['ssh'] + sshopts + ['--'] + [host_name] + list(args)

def check_output(*popenargs, **kwargs):
    r"""Run command with arguments and return its output as a byte string.

    Backported from Python 2.7 as it's implemented as pure python on stdlib.

    >>> check_output(['/usr/bin/python', '--version'])
    Python 2.6.2
    """
    process = subprocess.Popen(stdout=subprocess.PIPE, *popenargs, **kwargs)
    output, unused_err = process.communicate()
    retcode = process.poll()
    if retcode:
        cmd = kwargs.get("args")
        if cmd is None:
            cmd = popenargs[0]
        error = subprocess.CalledProcessError(retcode, cmd)
        error.output = output
        raise error
    return output



# -*- coding: utf-8 -*-
"""
    Copyright 2010 cloudControl UG (haftungsbeschraenkt)

    Licensed under the Apache License, Version 2.0 (the "License");
    you may not use this file except in compliance with the License.
    You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

    Unless required by applicable law or agreed to in writing, software
    distributed under the License is distributed on an "AS IS" BASIS,
    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
    See the License for the specific language governing permissions and
    limitations under the License.
"""
from __builtin__ import open, raw_input, range
from exceptions import ImportError, ValueError

from getpass import getpass
import sys
import os
from cctrl.oshelpers import recode_input

try:
    import json
except ImportError:
    import simplejson as json

from cctrl.error import messages, PasswordsDontMatchException
from cctrl.settings import TOKEN_FILE_PATH, HOME_PATH


def update_tokenfile(api):
    """
        Because it is a real pain we don't want to ask developers for their
        username and password every time they call a method.

        Therefore we authenticate users via token for each request and only
        require email and password for a new token.
        A token is valid for a given period of time. Each successful API
        request resets the expiration time.
    """
    if api.check_token():
        write_tokenfile(api)
        return True
    return False


def read_tokenfile():
    """
        Read the token from the token_file in TOKEN_FILE_PATH specified in
        cctrl.settings
    """
    token = None
    if os.path.exists(TOKEN_FILE_PATH):
        token_file = open(TOKEN_FILE_PATH, "r")
        try:
            token = json.load(token_file)
        except ValueError:
            token = None
        token_file.close()
    return token


def write_tokenfile(api):
    """
        This method checks, if the .cloudControl directory inside the
        users home exists or is a file. If not, we create it and then
        write the token file.
    """
    if os.path.isdir(HOME_PATH):
        pass
    elif os.path.isfile(HOME_PATH):
        print 'Error: ' + HOME_PATH + ' is a file, not a directory.'
        sys.exit(1)
    else:
        os.mkdir(HOME_PATH)

    tokenfile = open(TOKEN_FILE_PATH, "w")
    json.dump(api.get_token(), tokenfile)
    tokenfile.close()
    return True


def delete_tokenfile():
    """
        We delete the tokenfile if we don't have a valid token to save.
    """
    if os.path.exists(TOKEN_FILE_PATH):
        os.remove(TOKEN_FILE_PATH)
        return True
    return False


def get_credentials(create=False):
    """
        We use this to ask the user for his credentials in case we have no
        valid token.
        If create is true, the user is asked twice for the password,
        to make sure, that no typing error occurred. This is done three times
        after that a PasswordsDontMatchException is thrown.
    """
    sys.stderr.write('Email   : ')
    sys.stderr.flush()
    email = raw_input()
    password = None
    for i in range(3):
        password = recode_input(getpass('Password: '))
        if create:
            password2 = recode_input(getpass('Password (again): '))
            if password != password2:
                print messages['PasswordsDontMatch']
                if i == 2:
                    raise PasswordsDontMatchException()
            else:
                break
        else:
            break
    return email, password



# -*- coding: utf-8 -*-
"""
    setup script for cloudControl command line utilities

    usage: sudo python setup.py install
"""

import os
import sys
from cctrl.version import __version__

try:
    from setuptools import setup, find_packages
except ImportError:
    import ez_setup

    ez_setup.use_setuptools()
    from setuptools import setup, find_packages

execfile(os.path.join(os.path.dirname(os.path.realpath(__file__)), 'cctrl', 'version.py'))

if sys.version_info < (2, 6):
    required = ['simplejson']
else:
    required = []

required.append('pycclib>=1.5.4')
required.append('argparse>=1.1')

srcscripts = ['cctrl/cctrlapp', 'cctrl/cctrluser', 'cctrl/exoapp', 'cctrl/exouser', 'cctrl/dcapp', 'cctrl/dcuser', 'cctrl/cnhapp', 'cctrl/cnhuser']

if sys.platform == 'win32':
    import py2exe

    required.append('paramiko')
    required.append('ecdsa')
    extra_options = dict(
        setup_requires=['py2exe'],
        console=srcscripts,
        zipfile=None,
        data_files=[("", ["cctrl/cacerts.txt", ])],
        options={
            "py2exe": {
                "compressed": True,
                "optimize": 2,
                "excludes": [
                    '_scproxy',
                    'hexdump',
                    'isapi',
                    'pythoncom',
                    'pywintypes',
                    'simplejson',
                    'socks',
                    'win32com',
                    'win32com.client',
                    'doctest',
                    'pickle',
                    'difflib',
                    'unittest'],
                "includes": ["argparse", "pycclib", "paramiko", "Crypto", "ecdsa"],
                "packages": find_packages()
            }
        }
    )
else:
    extra_options = dict(
        scripts=srcscripts,
        package_data={"cctrl": ["cacerts.txt"]},
        packages=find_packages()
    )

setup(
    name="cctrl",
    version=__version__,
    description='cloudControl command line utilities',
    author='cloudControl Team',
    author_email='info@cloudcontrol.de',
    url='https://www.cloudcontrol.com',
    license='Apache 2.0',
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Environment :: Console',
        'Intended Audience :: Developers',
        'Intended Audience :: Information Technology',
        'Intended Audience :: System Administrators',
        'License :: OSI Approved :: Apache Software License',
        'Operating System :: MacOS :: MacOS X',
        'Operating System :: Microsoft :: Windows',
        'Operating System :: POSIX',
        'Programming Language :: Python',
        'Topic :: Internet'
    ],
    install_requires=required,
    tests_require=['mock'],
    test_suite='test',
    **extra_options
)

setup(
    name="dotcloudng",
    version=__version__,
    description='dotcloud command line utilities',
    author='dotcloud Team',
    author_email='info@dotcloud.com',
    url='https://www.dotcloud.com',
    license='Apache 2.0',
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Environment :: Console',
        'Intended Audience :: Developers',
        'Intended Audience :: Information Technology',
        'Intended Audience :: System Administrators',
        'License :: OSI Approved :: Apache Software License',
        'Operating System :: MacOS :: MacOS X',
        'Operating System :: Microsoft :: Windows',
        'Operating System :: POSIX',
        'Programming Language :: Python',
        'Topic :: Internet'
    ],
    install_requires=required,
    tests_require=['mock'],
    test_suite='test',
    **extra_options
)

setup(
    name="cnh",
    version=__version__,
    description='cloud&heat command line utilities',
    author='cloudControl Team',
    author_email='info@cloudcontrol.com',
    url='https://www.cloudcontrol.com',
    license='Apache 2.0',
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Environment :: Console',
        'Intended Audience :: Developers',
        'Intended Audience :: Information Technology',
        'Intended Audience :: System Administrators',
        'License :: OSI Approved :: Apache Software License',
        'Operating System :: MacOS :: MacOS X',
        'Operating System :: Microsoft :: Windows',
        'Operating System :: POSIX',
        'Programming Language :: Python',
        'Topic :: Internet'
    ],
    install_requires=required,
    tests_require=['mock'],
    test_suite='test',
    **extra_options
)
